@@ -1,4 +1,4 @@
-/* set-user--2.0.sql */
+/* set-user--3.0.sql */
 
 SET LOCAL search_path to @extschema@;
 
@@ -43,3 +43,11 @@ REVOKE EXECUTE ON FUNCTION @extschema@.set_user_u(text) FROM PUBLIC;
 /* No new sql functions for 1.5 */
 /* No new sql functions for 1.6 */
 /* No new sql functions for 2.0 */
+
+/* New functions in 3.0 begin here */
+
+CREATE FUNCTION @extschema@.set_session_auth(text)
+RETURNS text
+AS 'MODULE_PATHNAME', 'set_session_auth'
+LANGUAGE C STRICT;
+REVOKE EXECUTE ON FUNCTION @extschema@.set_session_auth(text) FROM PUBLIC;
